@@ -18,12 +18,12 @@
 */
 
 /*
- * $Id: scardcontrol.c,v 1.1.1.2 2004/08/15 20:52:06 rousseau Exp $
+ * $Id: scardcontrol.c,v 1.1.1.3 2005/03/15 18:55:25 rousseau Exp $
  */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
+#include <sys/time.h>
 #include <unistd.h>
 #include <string.h>
 #include <PCSC/winscard.h>
@@ -80,6 +80,9 @@ int main(int argc, char *argv[])
 	printf("SCardControl sample code\n");
 	printf("V 1.0 2004, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 
+	printf("\nTHIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL!\n");
+	printf("Do NOT use it unless you really know what you do.\n\n");
+
 	rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
 	if (rv != SCARD_S_SUCCESS)
 	{
@@ -235,6 +238,23 @@ int main(int argc, char *argv[])
 	/* verify PIN */
 	printf(" Secure verify PIN\n");
 	offset = 0;
+
+	/* CCID PIN verification data structure */
+	bSendBuffer[offset++] = 0x00;	/* bTimeOut */
+	bSendBuffer[offset++] = 0x82;	/* bmFormatString */
+	bSendBuffer[offset++] = 0x04;	/* bmPINBlockString (PIN length) */
+	bSendBuffer[offset++] = 0x00;	/* bmPINLengthFormat */
+	bSendBuffer[offset++] = 0x08;	/* wPINMaxExtraDigit: max */
+	bSendBuffer[offset++] = 0x04;	/* wPINMaxExtraDigit: min */
+	bSendBuffer[offset++] = 0x02;	/* bEntryValidationCondition */
+	bSendBuffer[offset++] = 0x00;	/* bNumberMessage */
+	bSendBuffer[offset++] = 0x04;	/* wLangId: english */
+	bSendBuffer[offset++] = 0x09;	/* " */
+	bSendBuffer[offset++] = 0x00;	/* bMsgIndex */
+	bSendBuffer[offset++] = 0x00;	/* bTeoPrologue */
+	bSendBuffer[offset++] = 0x00;	/* " */
+	bSendBuffer[offset++] = 0x00;	/* " */
+
 	/* APDU: 00 20 00 00 08 30 30 30 30 00 00 00 00 */
 	bSendBuffer[offset++] = 0x00;	/* CLA */
 	bSendBuffer[offset++] = 0x20;	/* INS: VERIFY */
@@ -250,22 +270,6 @@ int main(int argc, char *argv[])
 	bSendBuffer[offset++] = 0x00;	/* '\0' */
 	bSendBuffer[offset++] = 0x00;	/* '\0' */
 
-	/* CCID PIN verification data structure */
-	bSendBuffer[offset++] = 0x00;	/* bTimeOut */
-	bSendBuffer[offset++] = 0x82;	/* bmFormatString */
-	bSendBuffer[offset++] = 0x04;	/* bmPINBlockString (PIN length) */
-	bSendBuffer[offset++] = 0x00;	/* bmPINLengthFormat */
-	bSendBuffer[offset++] = 0x04;	/* wPINMaxExtraDigit: min */
-	bSendBuffer[offset++] = 0x04;	/* wPINMaxExtraDigit: max */
-	bSendBuffer[offset++] = 0x02;	/* bEntryValidationCondition */
-	bSendBuffer[offset++] = 0x00;	/* bNumberMessage */
-	bSendBuffer[offset++] = 0x04;	/* wLangId: english */
-	bSendBuffer[offset++] = 0x09;	/* " */
-	bSendBuffer[offset++] = 0x00;	/* bMsgIndex */
-	bSendBuffer[offset++] = 0x00;	/* bTeoPrologue */
-	bSendBuffer[offset++] = 0x00;	/* " */
-	bSendBuffer[offset++] = 0x00;	/* " */
-
 	printf(" command:");
 	for (i=0; i<offset; i++)
 		printf(" %02X", bSendBuffer[i]);
