@@ -2,23 +2,23 @@
     ccid.h: CCID structures
     Copyright (C) 2003   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
 /*
- * $Id: ccid.h,v 1.1.1.4 2004/08/15 20:52:06 rousseau Exp $
+ * $Id: ccid.h,v 1.1.1.5 2005/03/15 18:55:25 rousseau Exp $
  */
 
 typedef struct
@@ -80,8 +80,10 @@ typedef struct
 #define CCID_CLASS_AUTO_CONF_ATR	0x00000002
 #define CCID_CLASS_AUTO_VOLTAGE		0x00000008
 #define CCID_CLASS_AUTO_BAUD		0x00000020
+#define CCID_CLASS_AUTO_PPS_PROP	0x00000040
 #define CCID_CLASS_AUTO_PPS_CUR		0x00000080
 #define CCID_CLASS_AUTO_IFSD		0x00000400
+#define CCID_CLASS_CHARACTER		0x00000000
 #define CCID_CLASS_TPDU				0x00010000
 #define CCID_CLASS_SHORT_APDU		0x00020000
 #define CCID_CLASS_EXTENDED_APDU	0x00040000
@@ -92,6 +94,11 @@ typedef struct
 #define CCID_CLASS_PIN_MODIFY		0x02
 
 /* See CCID specs ch. 4.2.1 */
+#define CCID_ICC_PRESENT_ACTIVE		0x00	/* 00 0000 00 */
+#define CCID_ICC_PRESENT_INACTIVE	0x01	/* 00 0000 01 */
+#define CCID_ICC_ABSENT				0x02	/* 00 0000 20 */
+#define CCID_ICC_STATUS_MASK		0x03	/* 00 0000 11 */
+
 #define CCID_COMMAND_FAILED			0x40	/* 01 0000 00 */
 #define CCID_TIME_EXTENSION			0x80	/* 10 0000 00 */
 
@@ -102,6 +109,9 @@ typedef struct
 #define CARDMAN3121	0x076B3021
 #define LTC31		0x07830003
 #define SCR331DI	0x04E65111
+#define CHERRYXX33	0x046A0005
+#define OZ776		0x0B977762
+#define SPR532		0x04E6E003
 
 /* Escape sequence codes */
 #define ESC_GEMPC_SET_ISO_MODE		1
