@@ -4,23 +4,23 @@
  *
  * Thanks to Niki W. Waibel <niki.waibel@gmx.net> for a prototype version
  * 
- * This program is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
- * 
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License along 
- * with this program; if not, write to the Free Software Foundation, Inc., 
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  */
 
 /*
- * $Id: ccid_serial.c,v 1.1.1.5 2004/08/15 20:52:06 rousseau Exp $
+ * $Id: ccid_serial.c,v 1.1.1.6 2005/03/15 18:55:25 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -487,7 +487,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	int i;
 	unsigned int reader = reader_index;
 
-	DEBUG_COMM3("Reader index: %X, Device: %d", reader_index, dev_name);
+	DEBUG_COMM3("Reader index: %X, Device: %s", reader_index, dev_name);
 
 	/* check if the same channel is not already used */
 	for (i=0; i<CCID_DRIVER_MAX_READERS; i++)
@@ -570,7 +570,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	/* perform a command to be sure a GemPC Twin reader is connected
 	 * get the reader firmware */
 	{
-		unsigned char tx_buffer[] = "\x02";
+		unsigned char tx_buffer[] = { 0x02 };
 		unsigned char rx_buffer[50];
 		unsigned int rx_length = sizeof(rx_buffer);
 
