@@ -2,23 +2,23 @@
     ccid_usb.c: USB access routines using the libusb library
     Copyright (C) 2003-2004   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
 /*
- * $Id: ccid_usb.c,v 1.39 2004/08/06 11:18:41 rousseau Exp $
+ * $Id: ccid_usb.c,v 1.47 2005/03/14 18:56:51 rousseau Exp $
  */
 
 #define __CCID_USB__
@@ -58,12 +58,20 @@
  * Manufacturer and Product ID are also used to identify the device */
 #define ALLOW_PROPRIETARY_CLASS
 
+/*
+ * The O2Micro OZ776S reader has a wrong USB descriptor
+ * The extra[] field is associated with the last endpoint instead of the
+ * main USB descriptor
+ */
+#define O2MICRO_OZ776_PATCH
+
 #define BUS_DEVICE_STRSIZE 32
 
 typedef struct
 {
 	usb_dev_handle *handle;
 	struct usb_device *dev;
+	int interface;
 
 	/*
 	 * Endpoints
@@ -82,6 +90,7 @@ typedef struct
 #include "ccid_usb.h"
 
 static int get_end_points(struct usb_device *dev, _usbDevice *usb_device);
+int ccid_check_firmware(struct usb_device *dev);
 
 /* ne need to initialize to 0 since it is static */
 static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
@@ -90,6 +99,23 @@ static _usbDevice usbDevice[CCID_DRIVER_MAX_READERS];
 #define PCSCLITE_PRODKEY_NAME                   "ifdProductID"
 #define PCSCLITE_NAMEKEY_NAME                   "ifdFriendlyName"
 
+struct _bogus_firmware
+{
+	int vendor;		/* idVendor */
+	int product;	/* idProduct */
+	int firmware;	/* bcdDevice: previous firmwares have bugs */
+};
+
+static struct _bogus_firmware Bogus_firmwares[] = {
+	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */ /* the firmware version if not correct since I don't have received a working reader yet */
+	{ 0x04e6, 0xe001, 0x0516 },	/* SCR 331 */
+	{ 0x04e6, 0x5111, 0x0620 },	/* SCR 331-DI */
+	{ 0x04e6, 0x5115, 0x0514 },	/* SCR 335 */
+	{ 0x04e6, 0xe003, 0x0504 },	/* SPR 532 */
+	{ 0x0D46, 0x3001, 0x0037 },	/* KAAN Base */
+	{ 0x0D46, 0x3002, 0x0037 },	/* KAAN Advanced */
+};
+
 
 /*****************************************************************************
  *
@@ -351,12 +377,17 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					DEBUG_INFO3("Using USB bus/device: %s/%s",
 						 bus->dirname, dev->filename);
 
+					/* check for firmware bugs */
+					if (ccid_check_firmware(dev))
+						return STATUS_UNSUCCESSFUL;
+
 					/* Get Endpoints values*/
 					get_end_points(dev, &usbDevice[reader_index]);
 
 					/* store device information */
 					usbDevice[reader_index].handle = dev_handle;
 					usbDevice[reader_index].dev = dev;
+					usbDevice[reader_index].interface = interface;
 
 					/* CCID common informations */
 					usbDevice[reader_index].ccid.real_bSeq = 0;
@@ -469,9 +500,6 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
  ****************************************************************************/
 status_t CloseUSB(unsigned int reader_index)
 {
-	struct usb_interface *usb_interface;
-	int interface;
-
 	/* device not opened */
 	if (usbDevice[reader_index].dev == NULL)
 		return STATUS_UNSUCCESSFUL;
@@ -480,20 +508,17 @@ status_t CloseUSB(unsigned int reader_index)
 		usbDevice[reader_index].dev->bus->dirname,
 		usbDevice[reader_index].dev->filename);
 
-  	usb_interface = get_ccid_usb_interface(usbDevice[reader_index].dev);
-	interface = usb_interface ?
-		usb_interface->altsetting->bInterfaceNumber :
-		usbDevice[reader_index].dev->config->interface->altsetting->bInterfaceNumber;
-	
 	/* reset so that bSeq starts at 0 again */
 	usb_reset(usbDevice[reader_index].handle);
 
-	usb_release_interface(usbDevice[reader_index].handle, interface);
+	usb_release_interface(usbDevice[reader_index].handle,
+		usbDevice[reader_index].interface);
 	usb_close(usbDevice[reader_index].handle);
 
 	/* mark the resource unused */
 	usbDevice[reader_index].handle = NULL;
 	usbDevice[reader_index].dev = NULL;
+	usbDevice[reader_index].interface = 0;
 
 	return STATUS_SUCCESS;
 } /* CloseUSB */
@@ -515,16 +540,18 @@ _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index)
  *					get_desc
  *
  ****************************************************************************/
-int get_desc(int channel, usb_dev_handle **handle, struct
+int get_desc(int lun, usb_dev_handle **handle, struct
 	usb_device **dev)
 {
-	if (channel < 0 || channel > CCID_DRIVER_MAX_READERS)
-		return 1;
+	int reader_index;
 
-	*handle = usbDevice[channel].handle;
-	*dev = usbDevice[channel].dev;
+	if (-1 == (reader_index = LunToReaderIndex(lun)))
+		return FALSE;
 
-	return 0;
+	*handle = usbDevice[reader_index].handle;
+	*dev = usbDevice[reader_index].dev;
+
+	return TRUE;
 } /* get_desc */
 
 
@@ -588,9 +615,74 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usb_device)
 		}
 	}
 	else
-		/* we keep this in case a reader reports a bad class value */
+		/* only one interface found */
 		usb_interface = dev->config->interface;
 
+#ifdef O2MICRO_OZ776_PATCH
+	if (usb_interface != NULL
+		&& (OZ776 == (dev->descriptor.idVendor << 16)
+		+ dev->descriptor.idProduct)
+		&& (0 == usb_interface->altsetting->extralen)) /* this is the bug */
+	{
+		int i;
+
+		for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
+		{
+			/* find the extra[] array */
+			if (54 == usb_interface->altsetting->endpoint[i].extralen)
+			{
+				/* get the extra[] from the endpoint */
+				usb_interface->altsetting->extralen = 54;
+				usb_interface->altsetting->extra =
+					usb_interface->altsetting->endpoint[i].extra;
+				break;
+			}
+		}
+	}
+#endif
+
 	return usb_interface;
 } /* get_ccid_usb_interface */
 
+
+/*****************************************************************************
+ *
+ *					ccid_check_firmware
+ *
+ ****************************************************************************/
+int ccid_check_firmware(struct usb_device *dev)
+{
+	int i;
+
+	for (i=0; i<sizeof(Bogus_firmwares)/sizeof(Bogus_firmwares[0]); i++)
+	{
+		if (dev->descriptor.idVendor != Bogus_firmwares[i].vendor)
+				continue;
+
+		if (dev->descriptor.idProduct != Bogus_firmwares[i].product)
+			continue;
+
+		/* firmware too old and buggy */
+		if (dev->descriptor.bcdDevice < Bogus_firmwares[i].firmware)
+		{
+			if (DriverOptions & DRIVER_OPTION_USE_BOGUS_FIRMWARE)
+			{
+				DEBUG_INFO3("Firmware (%X.%02X) is bogus! but you choosed to use it",
+					dev->descriptor.bcdDevice >> 8,
+					dev->descriptor.bcdDevice & 0xFF);
+				return FALSE;
+			}
+			else
+			{
+				DEBUG_CRITICAL3("Firmware (%X.%02X) is bogus! Upgrade the reader firmware or get a new reader.",
+					dev->descriptor.bcdDevice >> 8,
+					dev->descriptor.bcdDevice & 0xFF);
+				return TRUE;
+			}
+		}
+	}
+
+	/* by default the firmware is not bogus */
+	return FALSE;
+} /* ccid_check_firmware */
+
