@@ -1,24 +1,24 @@
 /*
     commands.c: Commands sent to the card
-    Copyright (C) 2003   Ludovic Rousseau
+    Copyright (C) 2003-2004   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
 /*
- * $Id: commands.c,v 1.28 2004/08/06 11:13:25 rousseau Exp $
+ * $Id: commands.c,v 1.38 2005/03/14 18:56:51 rousseau Exp $
  */
 
 #include <string.h>
@@ -52,6 +52,10 @@ static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
 	unsigned char rx_buffer[]);
 
+static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index, unsigned int
+	tx_length, unsigned char tx_buffer[], unsigned int *rx_length, unsigned
+	char rx_buffer[]);
+
 static RESPONSECODE CmdXfrBlockTPDU_T1(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
 	unsigned char rx_buffer[]);
@@ -106,15 +110,15 @@ again:
 	{
 		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 
-		/* Protocol error in EMV mode */
-		if (buffer[ERROR_OFFSET] == 0xBB &&
-			ccid_descriptor->readerID == GEMPC433)
+		if (0xBB == buffer[ERROR_OFFSET] &&	/* Protocol error in EMV mode */
+			((GEMPC433 == ccid_descriptor->readerID)
+			|| (CHERRYXX33 == ccid_descriptor->readerID)))
 		{
-			unsigned char cmd[] = "\x1F\x01";
+			unsigned char cmd[] = {0x1F, 0x01};
 			unsigned char res[1];
 			unsigned int res_length = sizeof(res);
 
-			if ((return_value = CmdEscape(reader_index, cmd, sizeof(cmd)-1, res,
+			if ((return_value = CmdEscape(reader_index, cmd, sizeof(cmd), res,
 				&res_length)) != IFD_SUCCESS)
 				return return_value;
 
@@ -156,52 +160,12 @@ again:
 RESPONSECODE SecurePIN(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
-
 {
 	unsigned char cmd[11+14+CMD_BUF_SIZE];
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
-	int length = 0;
-
-	/* PIN verification data structure WITHOUT TeoPrologue */
-	if (TxBuffer[4] /* Lc */
-		+ 5 /* CLA, INS, P1, P2, Lc */
-		+ 11 /* CCID PIN verification data structure */ == TxLength)
-	{
-		i2dw(TxLength+3+1, cmd+1);	/* command length */
-
-		/* copy the CCID data structure */
-		memcpy(cmd +11, TxBuffer + TxBuffer[4] + 5, 11);
-
-		/* TeoPrologue not used */
-		memset(cmd +11 +11, 0, 3);
-
-		/* copy the APDU */
-		memcpy(cmd +11 +14, TxBuffer, TxLength-11);
-
-		length = 14 + TxLength;
-	}
-	/* PIN verification data structure WITH TeoPrologue */
-	else if (TxBuffer[4] /* Lc */
-		+ 5 /* CLA, INS, P1, P2, Lc */
-		+ 14 /* CCID PIN verification data structure */ == TxLength)
-	{
-		i2dw(TxLength+1, cmd+1);	/* command length */
-
-		/* copy the CCID data structure */
-		memcpy(cmd +11, TxBuffer + TxBuffer[4] + 5, 14);
-
-		/* copy the APDU */
-		memcpy(cmd +11 +14, TxBuffer, TxLength-14);
-
-		length = 11 + TxLength;
-	}
-	else
-	{
-		*RxLength = 0;
-		return IFD_COMMUNICATION_ERROR;
-	}
 
 	cmd[0] = 0x69;	/* Secure */
+	i2dw(TxLength+1, cmd+1);	/* command length (includes bPINOperation) */
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
 	cmd[7] = 0;		/* bBWI */
@@ -209,7 +173,32 @@ RESPONSECODE SecurePIN(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 0;	/* bPINOperation: PIN Verification */
 
-	if (WritePort(reader_index, length, cmd) != STATUS_SUCCESS)
+	/* check that the command is not too large */
+	if (TxLength > 14+CMD_BUF_SIZE)
+		return IFD_NOT_SUPPORTED;
+
+	/* CCID data structure + APDU */
+	memcpy(cmd + 11, TxBuffer, TxLength);
+
+	/* SPR532 and Case 1 APDU */
+	if ((SPR532 == ccid_descriptor->readerID) && (TxLength - 14 == 4))
+	{
+		RESPONSECODE return_value;
+		unsigned char cmd[] = { 0x80,0x02, 0x00 };
+		unsigned char res[1];
+		unsigned int res_length = sizeof(res);
+
+		/* the SPR532 will append the PIN code without any padding */
+		return_value = CmdEscape(reader_index, cmd, sizeof(cmd), res,
+			&res_length);
+		if (return_value != IFD_SUCCESS)
+		{
+			ccid_error(res[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);
+			return return_value;
+		}
+	}
+
+	if (WritePort(reader_index, TxLength+1+10, cmd) != STATUS_SUCCESS)
 		return IFD_COMMUNICATION_ERROR;
 
 	return CCID_Receive(reader_index, RxLength, RxBuffer);
@@ -355,7 +344,10 @@ RESPONSECODE CmdGetSlotStatus(unsigned int reader_index, unsigned char buffer[])
 	if (buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
-		return_value = IFD_COMMUNICATION_ERROR;
+
+		/* card absent or mute is not an communication error */
+		if (buffer[ERROR_OFFSET] != 0xFE)
+			return_value = IFD_COMMUNICATION_ERROR;
 	}
 
 	return return_value;
@@ -415,6 +407,18 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 				tx_length, tx_buffer, rx_length, rx_buffer);
 			break;
 
+		case CCID_CLASS_CHARACTER:
+			if (protocol == T_0)
+				return_value = CmdXfrBlockCHAR_T0(reader_index, tx_length,
+					tx_buffer, rx_length, rx_buffer);
+			else
+				if (protocol == T_1)
+					return_value = CmdXfrBlockTPDU_T1(reader_index, tx_length,
+						tx_buffer, rx_length, rx_buffer);
+ 				else
+					return_value = IFD_PROTOCOL_NOT_SUPPORTED;
+			break;
+
 		default:
 			*rx_length = 0;
 			return_value = IFD_COMMUNICATION_ERROR;
@@ -431,7 +435,7 @@ clean_up_and_return:
  *
  ****************************************************************************/
 RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
-	const unsigned char tx_buffer[], unsigned char bBWI)
+	const unsigned char tx_buffer[], unsigned short rx_length, unsigned char bBWI)
 {
 	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
@@ -441,7 +445,13 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
 	cmd[7] = bBWI;	/* extend block waiting timeout */
-	cmd[8] = cmd[9] = 0; /* RFU */
+	cmd[8] = rx_length & 0xFF;	/* Expected length */
+	cmd[9] = (rx_length >> 8) & 0xFF;
+
+	/* check that the command is not too large */
+	if (tx_length > CMD_BUF_SIZE)
+		return IFD_NOT_SUPPORTED;
+
 	memcpy(cmd+10, tx_buffer, tx_length);
 
 	if (WritePort(reader_index, 10+tx_length, cmd) != STATUS_SUCCESS)
@@ -510,14 +520,334 @@ static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
 
 	DEBUG_COMM2("T=0: %d bytes", tx_length);
 
-	return_value = CCID_Transmit(reader_index, tx_length, tx_buffer, 0);
+	return_value = CCID_Transmit(reader_index, tx_length, tx_buffer, 0, 0);
 	if (return_value != IFD_SUCCESS)
 		return return_value;
-	
+
 	return CCID_Receive(reader_index, rx_length, rx_buffer);
 } /* CmdXfrBlockTPDU_T0 */
 
 
+/*****************************************************************************
+ *
+ *					T0CmdParsing
+ *
+ ****************************************************************************/
+static RESPONSECODE T0CmdParsing(unsigned char *cmd, unsigned int cmd_len,
+	unsigned int *exp_len)
+{
+	*exp_len = 0;
+
+	/* Ref: 7816-4 Annex A */
+	switch (cmd_len)
+	{
+		case 4:	/* Case 1 */
+			*exp_len = 2; /* SW1 and SW2 only */
+			break;
+
+		case 5: /* Case 2 */
+			if (cmd[4] != 0)
+				*exp_len = cmd[4] + 2;
+			else
+				*exp_len = 256 + 2;
+			break;
+
+		default: /* Case 3 */
+			if (cmd_len > 5 && cmd_len == (unsigned int)(cmd[4] + 5))
+				*exp_len = 2; /* SW1 and SW2 only */
+			else
+				return IFD_COMMUNICATION_ERROR;	/* situation not supported */
+			break;
+	}
+
+	return IFD_SUCCESS;
+} /* T0CmdParsing */
+
+
+/*****************************************************************************
+ *
+ *					T0ProcACK
+ *
+ ****************************************************************************/
+static RESPONSECODE T0ProcACK(unsigned int reader_index,
+	unsigned char **snd_buf, unsigned int *snd_len,
+	unsigned char **rcv_buf, unsigned int *rcv_len,
+	unsigned char **in_buf, unsigned int *in_len,
+	unsigned int proc_len, int is_rcv)
+{
+	RESPONSECODE return_value;
+	unsigned int remain_len;
+	unsigned char tmp_buf[512];
+	unsigned int ret_len;
+
+	DEBUG_COMM2("Enter, is_rcv = %d", is_rcv);
+
+	if (is_rcv == 1)
+	{	/* Receiving mode */
+		if (*in_len > 0)
+		{	/* There are still available data in our buffer */
+			if (*in_len >= proc_len)
+			{
+				/* We only need to get the data from our buffer */
+				memcpy(*rcv_buf, *in_buf, proc_len);
+				*rcv_buf += proc_len;
+				*in_buf += proc_len;
+				*rcv_len += proc_len;
+				*in_len -= proc_len;
+
+				return IFD_SUCCESS;
+			}
+			else
+			{
+				/* Move all data in the input buffer to the reply buffer */
+				remain_len = proc_len - *in_len;
+				memcpy(*rcv_buf, *in_buf, *in_len);
+				*rcv_buf += *in_len;
+				*in_buf += *in_len;
+				*rcv_len += *in_len;
+				*in_len = 0;
+			}
+		}
+		else
+			/* There is no data in our tmp_buf,
+			 * we have to read all data we needed */
+			remain_len = proc_len;
+
+		/* Read the expected data from the smartcard */
+		if (*in_len != 0)
+		{
+			DEBUG_CRITICAL("*in_len != 0");
+			return IFD_COMMUNICATION_ERROR;
+		}
+
+		memset(tmp_buf, 0, sizeof(tmp_buf));
+
+		ret_len = remain_len;
+		return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+		if (return_value != IFD_SUCCESS)
+			return return_value;
+
+		return_value = CCID_Receive(reader_index, &ret_len, tmp_buf);
+		if (return_value != IFD_SUCCESS)
+			return return_value;
+
+		memcpy(*rcv_buf, tmp_buf, remain_len);
+		*rcv_buf += remain_len, *rcv_len += remain_len;
+
+		/* If ret_len != remain_len, our logic is erroneous */
+		if (ret_len != remain_len)
+		{
+			DEBUG_CRITICAL("ret_len != remain_len");
+			return IFD_COMMUNICATION_ERROR;
+		}
+	}
+	else
+	{	/* Sending mode */
+
+		return_value = CCID_Transmit(reader_index, proc_len, *snd_buf, 1, 0);
+		if (return_value != IFD_SUCCESS)
+			return return_value;
+
+		*snd_len -= proc_len;
+		*snd_buf += proc_len;
+	}
+
+	DEBUG_COMM("Exit");
+
+	return IFD_SUCCESS;
+} /* T0ProcACK */
+
+
+/*****************************************************************************
+ *
+ *					T0ProcSW1
+ *
+ ****************************************************************************/
+static RESPONSECODE T0ProcSW1(unsigned int reader_index,
+	unsigned char *rcv_buf, unsigned int *rcv_len,
+	unsigned char *in_buf, unsigned int in_len)
+{
+	RESPONSECODE return_value = IFD_SUCCESS;
+	UCHAR tmp_buf[512];
+	unsigned char *rcv_buf_tmp = rcv_buf;
+	const unsigned int rcv_len_tmp = *rcv_len;
+	unsigned char sw1, sw2;
+
+	/* store the SW1 */
+	sw1 = *rcv_buf = *in_buf;
+	rcv_buf++;
+	in_buf++;
+	in_len--;
+	(*rcv_len)++;
+
+	/* store the SW2 */
+	if (0 == in_len)
+	{
+		return_value = CCID_Transmit(reader_index, 0, rcv_buf, 1, 0);
+		if (return_value != IFD_SUCCESS)
+			return return_value;
+
+		in_len = 1;
+
+		return_value = CCID_Receive(reader_index, &in_len, tmp_buf);
+		if (return_value != IFD_SUCCESS)
+			return return_value;
+
+		in_buf = tmp_buf;
+	}
+	sw2 = *rcv_buf = *in_buf;
+	rcv_buf++;
+	in_buf++;
+	in_len--;
+	(*rcv_len)++;
+
+	if (return_value != IFD_SUCCESS)
+	{
+		rcv_buf_tmp[0] = rcv_buf_tmp[1] = 0;
+		*rcv_len = rcv_len_tmp;
+	}
+
+	DEBUG_COMM3("Exit: SW=%02X %02X", sw1, sw2);
+
+	return return_value;
+} /* T0ProcSW1 */
+
+
+/*****************************************************************************
+ *
+ *					CmdXfrBlockCHAR_T0
+ *
+ ****************************************************************************/
+static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
+	unsigned int snd_len, unsigned char snd_buf[], unsigned int *rcv_len,
+	unsigned char rcv_buf[])
+{
+	int is_rcv;
+	unsigned char cmd[5];
+	unsigned char tmp_buf[512];
+	unsigned int exp_len, in_len;
+	unsigned char ins, *in_buf;
+	RESPONSECODE return_value = IFD_SUCCESS;
+
+	DEBUG_COMM2("T=0: %d bytes", snd_len);
+
+	in_buf = tmp_buf;
+	in_len = 0;
+	*rcv_len = 0;
+
+	return_value = T0CmdParsing(snd_buf, snd_len, &exp_len);
+	if (return_value != IFD_SUCCESS)
+	{
+		DEBUG_CRITICAL("T0CmdParsing failed");
+		return IFD_COMMUNICATION_ERROR;
+	}
+
+	if (snd_len == 5 || snd_len == 4)
+		is_rcv = 1;
+	else
+		is_rcv = 0;
+
+	/* Command to send to the smart card (must be 5 bytes, from 7816 p.15) */
+	memset(cmd, 0, sizeof(cmd));
+	if (snd_len == 4)
+	{
+		memcpy(cmd, snd_buf, 4);
+		snd_buf += 4;
+		snd_len -= 4;
+	}
+	else
+	{
+		memcpy(cmd, snd_buf, 5);
+		snd_buf += 5;
+		snd_len -= 5;
+	}
+
+	/* Make sure this is a valid command by checking the INS field */
+	ins = cmd[1];
+	if ((ins & 0xF0) == 0x60 ||	/* 7816-3 8.3.2 */
+		(ins & 0xF0) == 0x90)
+	{
+		DEBUG_CRITICAL2("fatal: INS (0x%02X) = 0x6X or 0x9X", ins);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
+	return_value = CCID_Transmit(reader_index, 5, cmd, 1, 0);
+	if (return_value != IFD_SUCCESS)
+		return return_value;
+
+	while (1)
+	{
+		if (in_len == 0)
+		{
+			in_len = 1;
+			return_value = CCID_Receive(reader_index, &in_len, tmp_buf);
+			if (return_value != IFD_SUCCESS)
+			{
+				DEBUG_CRITICAL("CCID_Receive failed");
+				return return_value;
+			}
+			in_buf = tmp_buf;
+		}
+		if (in_len == 0)
+		{
+			/* Suppose we should be able to get data.
+			 * If not, error. Set the time-out error */
+			DEBUG_CRITICAL("error: in_len = 0");
+			return IFD_RESPONSE_TIMEOUT;
+		}
+
+		/* Start to process the procedure bytes */
+		if (*in_buf == 0x60)
+		{
+			in_len = 0;
+			return_value = CCID_Transmit(reader_index, 0, cmd, 1, 0);
+
+			if (return_value != IFD_SUCCESS)
+				return return_value;
+
+			continue;
+		}
+		else if (*in_buf == ins || *in_buf == (ins ^ 0x01))
+		{
+			/* ACK => To transfer all remaining data bytes */
+			in_buf++, in_len--;
+			if (is_rcv)
+				return_value = T0ProcACK(reader_index, &snd_buf, &snd_len,
+					&rcv_buf, rcv_len, &in_buf, &in_len, exp_len - *rcv_len, 1);
+			else
+				return_value = T0ProcACK(reader_index, &snd_buf, &snd_len,
+					&rcv_buf, rcv_len, &in_buf, &in_len, snd_len, 0);
+
+			if (*rcv_len == exp_len)
+				return return_value;
+
+			continue;
+		}
+		else if (*in_buf == (ins ^ 0xFF) || *in_buf == (ins ^ 0xFE))
+		{
+			/* ACK => To transfer 1 remaining bytes */
+			in_buf++, in_len--;
+			return_value = T0ProcACK(reader_index, &snd_buf, &snd_len,
+				&rcv_buf, rcv_len, &in_buf, &in_len, 1, is_rcv);
+
+			if (return_value != IFD_SUCCESS)
+				return return_value;
+
+			continue;
+		}
+		else if ((*in_buf & 0xF0) == 0x60 || (*in_buf & 0xF0) == 0x90)
+			/* SW1 */
+			return T0ProcSW1(reader_index, rcv_buf, rcv_len, in_buf, in_len);
+
+		/* Error, unrecognized situation found */
+		DEBUG_CRITICAL2("Unrecognized Procedure byte (0x%02X) found!", *in_buf);
+		return return_value;
+	}
+
+	return return_value;
+} /* CmdXfrBlockCHAR_T0 */
+
+
 /*****************************************************************************
  *
  *					CmdXfrBlockTPDU_T1
@@ -566,6 +896,11 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
 	cmd[7] = protocol;	/* bProtocolNum */
 	cmd[8] = cmd[9] = 0; /* RFU */
+
+	/* check that the command is not too large */
+	if (length > CMD_BUF_SIZE)
+		return IFD_NOT_SUPPORTED;
+
 	memcpy(cmd+10, buffer, length);
 
 	if (WritePort(reader_index, 10+length, cmd) != STATUS_SUCCESS)
@@ -578,13 +913,27 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
-		return IFD_COMMUNICATION_ERROR;
+		if (0x00 == cmd[ERROR_OFFSET])	/* command not supported */
+			return IFD_NOT_SUPPORTED;
+		else
+			return IFD_COMMUNICATION_ERROR;
 	}
 
 	return IFD_SUCCESS;
 } /* SetParameters */
 
 
+/*****************************************************************************
+ *
+ *					isCharLevel
+ *
+ ****************************************************************************/
+int isCharLevel(int reader_index)
+{
+	return CCID_CLASS_CHARACTER == (get_ccid_descriptor(reader_index)->dwFeatures & CCID_CLASS_EXCHANGE_MASK);
+} /* isCharLevel */
+
+
 /*****************************************************************************
  *
  *					i2dw
