@@ -1,24 +1,24 @@
 /*
     debug.c: log (or not) messages
-    Copyright (C) 2003-2004   Ludovic Rousseau
+    Copyright (C) 2003-2005   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
 /*
- * $Id: debug.c,v 1.3 2004/05/24 12:58:43 rousseau Exp $
+ * $Id: debug.c,v 1.5 2005/03/14 18:56:52 rousseau Exp $
  */
 
 
@@ -33,6 +33,8 @@
 
 static char DebugBuffer[DEBUG_BUF_SIZE];
 
+#define LOG_TO_STDERR
+
 void debug_msg(char *fmt, ...)
 {
 	va_list argptr;
@@ -41,7 +43,9 @@ void debug_msg(char *fmt, ...)
 	vsnprintf(DebugBuffer, DEBUG_BUF_SIZE, fmt, argptr);
 	va_end(argptr);
 
+#ifdef LOG_TO_STDERR
 	fprintf(stderr, "%s\n", DebugBuffer);
+#endif
 } /* debug_msg */
 
 void debug_xxd(const char *msg, const unsigned char *buffer, const int len)
@@ -60,9 +64,11 @@ void debug_xxd(const char *msg, const unsigned char *buffer, const int len)
 		c += strlen(c);
 	}
 
+#ifdef LOG_TO_STDERR
 	if (c >= debug_buf_end)
 		fprintf(stderr, "Debug buffer too short\n");
 
 	fprintf(stderr, "%s\n", DebugBuffer);
+#endif
 } /* debug_xxd */
 
