@@ -2,22 +2,22 @@
     ifdhandler.c: IFDH API
     Copyright (C) 2003   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c,v 1.50 2004/08/06 11:32:01 rousseau Exp $ */
+/* $Id: ifdhandler.c,v 1.59 2005/03/14 18:56:52 rousseau Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -361,7 +361,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	 */
 
 	BYTE pps[PPS_MAX_LENGTH];
-	ATR atr;
+	ATR_t atr;
 	unsigned int len;
 	int convention;
 	int reader_index;
@@ -383,6 +383,11 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	ccid_slot = get_ccid_slot(reader_index);
 	ccid_desc = get_ccid_descriptor(reader_index);
 
+	/* Do not send CCID command SetParameters or PPS to the CCID
+	 * The CCID will do this himself */
+	if (ccid_desc->dwFeatures & CCID_CLASS_AUTO_PPS_PROP)
+		return IFD_SUCCESS;
+
 	/* Get ATR of the card */
 	ATR_InitFromArray(&atr, ccid_slot->pcATRBuffer, ccid_slot->nATRLength);
 
@@ -446,9 +451,8 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	}
 	else
 	{
-		/* PPS not negociated by reader, and TA1 present */
-		if (atr.ib[0][ATR_INTERFACE_BYTE_TA].present &&
-			! (ccid_desc->dwFeatures & CCID_CLASS_AUTO_PPS_CUR))
+		/* TA1 present */
+		if (atr.ib[0][ATR_INTERFACE_BYTE_TA].present)
 		{
 			unsigned int card_baudrate;
 			unsigned int default_baudrate;
@@ -465,7 +469,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 				* ATR_DEFAULT_D / ATR_DEFAULT_F);
 
 			/* if the reader is fast enough */
-			if ((card_baudrate < ccid_desc->dwMaxDataRate)
+			if ((card_baudrate <= ccid_desc->dwMaxDataRate)
 				/* and the card does not try to lower the default speed */
 				&& (card_baudrate > default_baudrate ))
 			{
@@ -536,13 +540,20 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			0x00	/* NADValue		*/
 		};
 		int i;
+		t1_state_t *t1 = &(ccid_slot -> t1);
+		RESPONSECODE ret;
 
 		/* TA1 is not default */
 		if (PPS_HAS_PPS1(pps))
 			param[0] = pps[2];
+ 
+		/* CRC checksum? */
+		if (2 == t1->rc_bytes)
+			param[1] |= 0x01;
 
+		/* the CCID should ignore this bit */
 		if (ATR_CONVENTION_INVERSE == convention)
-			param[1] &= 0x02;
+			param[1] |= 0x02;
 
 		/* get TC1 Extra guard time */
 		if (atr.ib[0][ATR_INTERFACE_BYTE_TC].present)
@@ -560,8 +571,9 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 				break;
 			}
 
-		if (IFD_SUCCESS != SetParameters(reader_index, 1, sizeof(param), param))
-			return IFD_COMMUNICATION_ERROR;
+		ret = SetParameters(reader_index, 1, sizeof(param), param);
+		if (IFD_SUCCESS != ret)
+			return ret;
 	}
 	else
 	/* T=0 */
@@ -573,13 +585,14 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			0x0A,	/* WaitingInteger	*/
 			0x00	/* ClockStop		*/
 		};
+		RESPONSECODE ret;
 
 		/* TA1 is not default */
 		if (PPS_HAS_PPS1(pps))
 			param[0] = pps[2];
 
 		if (ATR_CONVENTION_INVERSE == convention)
-			param[1] &= 0x02;
+			param[1] |= 0x02;
 
 		/* get TC1 Extra guard time */
 		if (atr.ib[0][ATR_INTERFACE_BYTE_TC].present)
@@ -589,8 +602,9 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		if (atr.ib[1][ATR_INTERFACE_BYTE_TC].present)
 			param[3] = atr.ib[1][ATR_INTERFACE_BYTE_TC].value;
 
-		if (IFD_SUCCESS != SetParameters(reader_index, 0, sizeof(param), param))
-			return IFD_COMMUNICATION_ERROR;
+		ret = SetParameters(reader_index, 0, sizeof(param), param);
+		if (IFD_SUCCESS != ret)
+			return ret;
 	}
 
 	/* set IFSC & IFSD in T=1 */
@@ -895,15 +909,15 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 		return IFD_COMMUNICATION_ERROR;
 
 	return_value = IFD_COMMUNICATION_ERROR;
-	switch (pcbuffer[7])	/* bStatus */
+	switch (pcbuffer[7] & CCID_ICC_STATUS_MASK)	/* bStatus */
 	{
-		case 0x00:
-		case 0x01:
+		case CCID_ICC_PRESENT_ACTIVE:
+		case CCID_ICC_PRESENT_INACTIVE:
 			return_value = IFD_ICC_PRESENT;
 			/* use default slot */
 			break;
 
-		case 0x02:
+		case CCID_ICC_ABSENT:
 			/* Reset ATR buffer */
 			CcidSlots[reader_index].nATRLength = 0;
 			*CcidSlots[reader_index].pcATRBuffer = '\0';
