@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: parse.c,v 1.18 2004/08/06 13:29:00 rousseau Exp $
+ * $Id: parse.c,v 1.23 2005/02/25 17:26:57 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -65,8 +65,10 @@ int main(/*@unused@*/ int argc, /*@unused@*/ char *argv[])
 			usb_dev_handle *handle;
 			struct usb_device *dev;
 
-			get_desc(channel, &handle, &dev);
-			res = ccid_parse_interface_descriptor(handle, dev);
+			if (get_desc(channel<<16, &handle, &dev))
+				res = ccid_parse_interface_descriptor(handle, dev);
+			else
+				fprintf(stderr, "Reader %d not found\n", channel);
 		}
 	}
 
@@ -101,17 +103,22 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	/*
 	 * Vendor/model name
 	 */
+	printf(" idVendor: 0x%04X\n", dev->descriptor.idVendor);
 	if (usb_get_string_simple(handle, dev->descriptor.iManufacturer,
 		buffer, sizeof(buffer)) < 0)
-		printf(" Can't get iManufacturer string\n");
+		printf("  Can't get iManufacturer string\n");
 	else
-		printf(" iManufacturer: %s\n", buffer);
+		printf("  iManufacturer: %s\n", buffer);
 
+	printf(" idProduct: 0x%04X\n", dev->descriptor.idProduct);
 	if (usb_get_string_simple(handle, dev->descriptor.iProduct,
 		buffer, sizeof(buffer)) < 0)
-		printf(" Can't get iProduct string\n");
+		printf("  Can't get iProduct string\n");
 	else
-		printf(" iProduct: %s\n", buffer);
+		printf("  iProduct: %s\n", buffer);
+
+	printf(" bcdDevice: %X.%02X (firmware release?)\n",
+		dev->descriptor.bcdDevice >> 8, dev->descriptor.bcdDevice & 0xFF);
 
 	usb_interface = get_ccid_usb_interface(dev)->altsetting;
 	
@@ -232,21 +239,33 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	if (extra[40] & 0x20)
 		printf("   Automatic baud rate change according to frequency and Fi, Di parameters\n");
 	if (extra[40] & 0x40)
-		printf("   Automatic parameters negotiation made by the ICC\n");
+		printf("   Automatic parameters negotiation made by the CCID\n");
 	if (extra[40] & 0x80)
-		printf("   Automatic PPS made by the ICC\n");
+		printf("   Automatic PPS made by the CCID\n");
 	if (extra[41] & 0x01)
 		printf("   CCID can set ICC in clock stop mode\n");
 	if (extra[41] & 0x02)
 		printf("   NAD value other than 00 accepted (T=1)\n");
 	if (extra[41] & 0x04)
 		printf("   Automatic IFSD exchange as first exchange (T=1)\n");
-	if (extra[42] & 0x01)
-		printf("   TPDU level exchange\n");
-	if (extra[42] & 0x02)
-		printf("   Short APDU level exchange\n");
-	if (extra[42] & 0x04)
-		printf("   Short and Extended APDU level exchange\n");
+	switch (extra[42] & 0x07)
+	{
+		case 0x00:
+			printf("   Character level exchange\n");
+			break;
+
+		case 0x01:
+			printf("   TPDU level exchange\n");
+			break;
+
+		case 0x02:
+			printf("   Short APDU level exchange\n");
+			break;
+
+		case 0x04:
+			printf("   Short and Extended APDU level exchange\n");
+			break;
+	}
 
 	printf("  dwMaxCCIDMessageLength: %d bytes\n", dw2i(extra, 44));
 	printf("  bClassGetResponse: %d\n", extra[48]);
